"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.remove = exports.update = exports.create = exports.get = exports.getAll = exports.getArticlesByUserId = void 0;
const core_1 = require("@sequelize/core");
const Article_model_1 = require("../database/models/Article.model");
const File_model_1 = require("../database/models/File.model");
const Tag_model_1 = require("../database/models/Tag.model");
const User_model_1 = require("../database/models/User.model");
const tagService = require("./tag.service");
const includeArray = [
    {
        model: Tag_model_1.Tag,
        through: { attributes: [] },
        attributes: ["id", "name", "tagType"],
    },
    { model: File_model_1.File, attributes: ["id", "hash", "name", "altText"] },
    { model: User_model_1.User, attributes: ["id", "username"] },
];
async function getArticlesByUserId(userId) {
    const articles = await Article_model_1.Article.findAll({
        where: { authorId: userId },
        include: includeArray,
    });
    return articles;
}
exports.getArticlesByUserId = getArticlesByUserId;
async function getAll(searchString = undefined) {
    return new Promise(async (res, err) => {
        let articles;
        if (searchString) {
            articles = await Article_model_1.Article.findAll({
                include: includeArray,
                where: {
                    [core_1.Op.or]: {
                        title: { [core_1.Op.like]: `%${searchString}%` },
                        description: { [core_1.Op.like]: `%${searchString}%` },
                    },
                },
            });
        }
        else {
            articles = await Article_model_1.Article.findAll({
                include: includeArray,
            });
        }
        res(articles);
    });
}
exports.getAll = getAll;
async function get(articleId) {
    return new Promise(async (res, err) => {
        const article = await Article_model_1.Article.findByPk(articleId, {
            include: includeArray,
        });
        if (!article) {
            err("Could not find article");
            return;
        }
        res(article);
    });
}
exports.get = get;
async function create(title, description, authorId, tags, files) {
    return new Promise(async (res, error) => {
        const article = await Article_model_1.Article.create({
            title: title,
            description: description,
            authorId: authorId,
        });
        if (!article) {
            error("Could not create article");
        }
        await tagService.addArticleTags(article.id, tags);
        for (let file of files) {
            await File_model_1.File.create({
                articleId: article.id,
                name: file.name,
                hash: file.hash,
                altText: file.altText,
            });
        }
        get(article.id)
            .then((art) => {
            res(art);
        })
            .catch((err) => {
            error(err);
        });
    });
}
exports.create = create;
async function update(articleId, newTitle, newDescription, newTags, newFiles) {
    return new Promise(async (res, err) => {
        await Article_model_1.Article.update({
            title: newTitle,
            description: newDescription,
        }, { where: { id: articleId } });
        if (newTags) {
            await tagService.clearArticleTags(articleId);
            await tagService.addArticleTags(articleId, newTags);
        }
        if (newFiles) {
            await File_model_1.File.destroy({ where: { articleId: articleId } });
            for (let file of newFiles) {
                await File_model_1.File.create({
                    articleId: articleId,
                    name: file.name,
                    hash: file.hash,
                    altText: file.altText,
                });
            }
        }
        const updatedArticle = await Article_model_1.Article.findByPk(articleId);
        if (!updatedArticle) {
            err("Could not get article after updating");
            return;
        }
        res(updatedArticle);
    });
}
exports.update = update;
async function remove(articleId) {
    return new Promise(async (res, err) => {
        const affectedRows = await Article_model_1.Article.destroy({
            where: { id: articleId },
        });
        if (affectedRows < 1) {
            err("Could not delete row");
            return;
        }
        res(articleId);
    });
}
exports.remove = remove;
